# sage_setup: distribution = sagemath-repl
# sage.doctest: needs fpylll sage.rings.number_field sage.symbolic
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 21.6 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.<a> = QQ[sqrt(2)+sqrt(3)]; M
    Number Field in a with defining polynomial x^4 - 10*x^2 + 1 with a = 3.146264369941973?

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F.<y> = QQ[]
    sage: p = y^3 - 1/4*y^2 - 1/16*y + 1/4
    sage: p.is_irreducible()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: N.<b> = NumberField(p, 'b'); N
    Number Field in b with
    defining polynomial y^3 - 1/4*y^2 - 1/16*y + 1/4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: z = polygen(QQ, 'z')
    sage: q = z^3 - 1/4*z^2 - 1/16*z + 1/4
    sage: q.parent()
    Univariate Polynomial Ring in z over Rational Field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: P.<c> = NumberField(q, 'c'); P
    Number Field in c with
    defining polynomial z^3 - 1/4*z^2 - 1/16*z + 1/4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.polynomial()
    x^4 - 10*x^2 + 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: N.polynomial()
    y^3 - 1/4*y^2 - 1/16*y + 1/4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: element = -b^2 + 1/3*b + 4
    sage: element.parent()
    Number Field in b with
    defining polynomial y^3 - 1/4*y^2 - 1/16*y + 1/4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r = element.minpoly('t'); r
    t^3 - 571/48*t^2 + 108389/2304*t - 13345/216

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r.parent()
    Univariate Polynomial Ring in t over Rational Field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r.subs(t=element)
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.<a,b> = QQ[sqrt(2), sqrt(3)]
    sage: A
    Number Field in sqrt2 with defining polynomial x^2 - 2 over
    its base field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: B = A.base_field(); B
    Number Field in sqrt3 with defining polynomial x^2 - 3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.is_relative()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: B.is_relative()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: C.<c> = A.absolute_field()
    sage: C
    Number Field in c with defining polynomial x^4 - 10*x^2 + 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromC, toC = C.structure()
    sage: fromC(c)
    sqrt2 - sqrt3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: toC(a)
    1/2*c^3 - 9/2*c

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: toC(b)
    1/2*c^3 - 11/2*c

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: B.degree()
    2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.absolute_degree()
    4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.relative_degree()
    2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: x = polygen(QQ, 'x')
    sage: p = x^4 + x^2 - 1
    sage: p.parent()
    Univariate Polynomial Ring in x over Rational Field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: p.is_irreducible()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.<a> = NumberField(p, 'a')
    sage: y = polygen(M, 'y')
    sage: p = p.subs(x = y)
    sage: p
    y^4 + y^2 - 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: p.parent()
    Univariate Polynomial Ring in y over Number Field in a with
    defining polynomial x^4 + x^2 - 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: p.factor()
    (y - a) * (y + a) * (y^2 + a^2 + 1)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a^2 + 1 in QQ
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: q = y^2 + a^2 + 1
    sage: N.<b> = NumberField(q, 'b')
    sage: R.<z> = N[]
    sage: s = R(p)
    sage: s
    z^4 + z^2 - 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: s.parent()
    Univariate Polynomial Ring in z over Number Field in b with
    defining polynomial y^2 + a^2 + 1 over its base field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: s.factor()
    (z + b) * (z + a) * (z - a) * (z - b)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a in N, b in N
    (True, True)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: P.<c> = N.absolute_field()
    sage: w = polygen(P, 'w')
    sage: p = w^4 + w^2- 1
    sage: p.factor()
    (w - 7/18966*c^7 + 110/9483*c^5 +  923/9483*c^3 +  3001/6322*c) *
    (w - 7/37932*c^7 +  55/9483*c^5 + 923/18966*c^3 - 3321/12644*c) *
    (w + 7/37932*c^7 -  55/9483*c^5 - 923/18966*c^3 + 3321/12644*c) *
    (w + 7/18966*c^7 - 110/9483*c^5 -  923/9483*c^3 -  3001/6322*c)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromP, toP = P.structure()
    sage: fromP(7/18966*c^7 - 110/9483*c^5 - 923/9483*c^3 - 3001/6322*c)
    -b

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.degree()
    4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: N.relative_degree()
    2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: P.degree()
    8

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.degree()*N.relative_degree() == P.degree()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: x = polygen(QQ, 'x')
    sage: p = x^4 + x^2 - 1
    sage: r = p.roots(ring=QQbar); r
    [(-0.7861513777574233?,  1), (0.7861513777574233?,  1),
     (-1.272019649514069?*I, 1), (1.272019649514069?*I, 1)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r1 = r[0][0]; r1
    -0.7861513777574233?

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r1.as_number_field_element()
    (Number Field in a with defining polynomial y^4 - y^2 - 1,
     a^3 - a,
     Ring morphism:
       From: Number Field in a with defining polynomial y^4 - y^2 - 1
       To:   Algebraic Real Field
       Defn: a |--> -1.272019649514069?)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r1^4 + r1^2 - 1
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: N, rexact, homomorphism = r1.as_number_field_element()
    sage: (rexact)^4 + rexact^2 - 1
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: log(QQ[cos(pi/9)].degree(), 2) in ZZ
    False
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.misc.cython
"""
Cython support functions

AUTHORS:

- William Stein (2006-01-18): initial version
- William Stein (2007-07-28): update from sagex to cython
- Martin Albrecht & William Stein (2011-08): cfile & cargs
"""

# ****************************************************************************
#       Copyright (C) 2006 William Stein <wstein@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

import builtins
import os
import re
import sys
import shutil
import webbrowser
from pathlib import Path

from sage.env import (SAGE_LOCAL, cython_aliases,
                      sage_include_directories)
from sage.misc.cachefunc import cached_function
from sage.misc.sage_ostools import restore_cwd, redirection
from sage.misc.temporary_file import spyx_tmp, tmp_filename
from sage.repl.user_globals import get_globals


@cached_function
def _standard_libs_libdirs_incdirs_aliases():
    r"""
    Return the list of libraries and library directories.

    EXAMPLES::

        sage: from sage.misc.cython import _standard_libs_libdirs_incdirs_aliases
        sage: _standard_libs_libdirs_incdirs_aliases()
        (['mpfr', 'gmp', 'gmpxx', 'pari', ...],
         [...],
         [...],
         {...})
    """
    aliases = cython_aliases()
    standard_libs = [
        'mpfr', 'gmp', 'gmpxx', 'pari', 'm',
        'ec', 'gsl',
    ] + aliases["CBLAS_LIBRARIES"] + [
        'ntl']
    standard_libdirs = []
    if SAGE_LOCAL:
        standard_libdirs.append(os.path.join(SAGE_LOCAL, "lib"))
    standard_libdirs.extend(aliases["CBLAS_LIBDIR"] + aliases["NTL_LIBDIR"])
    standard_incdirs = sage_include_directories(use_sources=True) + aliases["CBLAS_INCDIR"] + aliases["NTL_INCDIR"]
    return standard_libs, standard_libdirs, standard_incdirs, aliases

################################################################
# If the user attaches a .spyx file and changes it, we have
# to reload an .so.
#
# PROBLEM: Python does not allow one to reload an .so extension module.
# Solution, we create a different .so file and load that one,
# overwriting the definitions of everything in the original .so file.
#
# HOW: By using a sequence_number for each .spyx file; we keep
# these sequence numbers in a dict.
#
################################################################


sequence_number = {}


def _webbrowser_open_file(path):
    """
    Open a html file in a web browser.
    """
    webbrowser.open(Path(path).as_uri())


def cython(filename, verbose=0, compile_message=False,
           use_cache=False, create_local_c_file=False, annotate=True, view_annotate=False,
           view_annotate_callback=None, sage_namespace=True, create_local_so_file=False):
    r"""
    Compile a Cython file. This converts a Cython file to a C (or C++ file),
    and then compiles that. The .c file and the .so file are
    created in a temporary directory.

    INPUT:

    - ``filename`` -- the name of the file to be compiled; should end with
      'pyx'

    - ``verbose`` -- integer (default: 0); level of verbosity. A negative
      value ensures complete silence.

    - ``compile_message`` -- boolean (default: ``False``); if ``True``, print
      ``'Compiling <filename>...'`` to the standard error

    - ``use_cache`` -- boolean (default: ``False``); if ``True``, check the
      temporary build directory to see if there is already a
      corresponding .so file. If so, and if the .so file is newer than the
      Cython file, don't recompile, just reuse the .so file.

    - ``create_local_c_file`` -- boolean (default: ``False``); if ``True``, save a
      copy of the ``.c`` or ``.cpp`` file in the current directory

    - ``annotate`` -- boolean (default: ``True``); if ``True``, create an html file which
      annotates the conversion from .pyx to .c. By default this is only created
      in the temporary directory, but if ``create_local_c_file`` is also True,
      then save a copy of the .html file in the current directory.

    - ``view_annotate`` -- boolean (default: ``False``); if ``True``, open the
      annotated html file in a web browser

    - ``view_annotate_callback`` -- function; a function that takes a string
      being the path to the html file. This can be overridden to change
      what to do with the annotated html file. Have no effect unless
      ``view_annotate`` is ``True``. By default, the html file is opened in a
      web browser.

    - ``sage_namespace`` -- boolean (default: ``True``); if ``True``, import
      ``sage.all``

    - ``create_local_so_file`` -- boolean (default: ``False``); if ``True``, save a
      copy of the compiled .so file in the current directory

    OUTPUT: a tuple ``(name, dir)`` where ``name`` is the name
    of the compiled module and ``dir`` is the directory containing
    the generated files.

    TESTS:

    Before :issue:`12975`, it would have been needed to write ``#clang c++``,
    but upper case ``C++`` has resulted in an error.
    Using pkgconfig to find the libraries, headers and macros. This is a
    work around while waiting for :issue:`22461` which will offer a better
    solution::

        sage: code = [
        ....: "#clang C++",
        ....: "from sage.rings.polynomial.multi_polynomial_libsingular cimport MPolynomial_libsingular",
        ....: "from sage.libs.singular.polynomial cimport singular_polynomial_pow",
        ....: "def test(MPolynomial_libsingular p):",
        ....: "    singular_polynomial_pow(&p._poly, p._poly, 2, p._parent_ring)"]
        sage: cython(os.linesep.join(code))

    The function ``test`` now manipulates internal C data of polynomials,
    squaring them::

        sage: P.<x,y>=QQ[]
        sage: test(x)
        sage: x
        x^2

    Check that compiling C++ code works::

        sage: cython("# distutils: language = c++\n"+
        ....:        "from libcpp.vector cimport vector\n"
        ....:        "cdef vector[int] * v = new vector[int](4)\n")

    Check that compiling C++ code works when creating a local C file,
    first moving to a tempdir to avoid clutter.  Before :issue:`22113`,
    the create_local_c_file argument was not tested for C++ code::

        sage: orig_cwd = os.getcwd()
        sage: import tempfile
        sage: with tempfile.TemporaryDirectory() as d:
        ....:     os.chdir(d)
        ....:     with open("test.pyx", 'w') as f:
        ....:         _ = f.write("# distutils: language = c++\n"
        ....:           "from libcpp.vector cimport vector\n"
        ....:           "cdef vector[int] * v = new vector[int](4)\n")
        ....:     output = sage.misc.cython.cython("test.pyx",
        ....:                                      create_local_c_file=True)
        ....:     os.chdir(orig_cwd)

    Accessing a ``.pxd`` file from the current directory works::

        sage: orig_cwd = os.getcwd()
        sage: import tempfile
        sage: with tempfile.TemporaryDirectory() as d:
        ....:     os.chdir(d)
        ....:     with open("helper.pxd", 'w') as f:
        ....:         _ = f.write("cdef inline int the_answer(): return 42")
        ....:     cython(
        ....:           "from helper cimport the_answer\n"
        ....:           "print(the_answer())"
        ....:     )
        ....:     os.chdir(orig_cwd)
        42

    Warning and error messages generated by Cython are properly
    handled. Warnings are only shown if verbose >= 0::

        sage: code = '''
        ....: def test_unreachable():
        ....:     raise Exception
        ....:     return 42
        ....: '''
        sage: cython(code, verbose=-1)
        sage: cython(code, verbose=0)
        warning: ...:4:4: Unreachable code...

        sage: cython("foo = bar\n")
        Traceback (most recent call last):
        ...
        RuntimeError: Error compiling Cython file:
        ------------------------------------------------------------
        ...
        foo = bar
             ^
        ------------------------------------------------------------
        <BLANKLINE>
        ...:1:6: undeclared name not builtin: bar

        sage: cython("cdef extern from 'no_such_header_file': pass")
        Traceback (most recent call last):
        ...
        RuntimeError: ...

    As of :issue:`29139` the default is ``cdivision=True``::

        sage: cython('''
        ....: cdef size_t foo = 3/2
        ....: ''')

    Check that Cython supports PEP 420 packages::

        sage: cython('''
        ....: cimport sage.misc.cachefunc
        ....: ''')

        sage: cython('''
        ....: from sage.misc.cachefunc cimport cache_key
        ....: ''')

    Test ``view_annotate``::

        sage: cython('''                                        # optional -- webbrowser
        ....: def f(int n):
        ....:     return n*n
        ....: ''', view_annotate=True)

    ::

        sage: cython('''
        ....: def f(int n):
        ....:     return n*n
        ....: ''', view_annotate=True, annotate=False)
        Traceback (most recent call last):
        ...
        ValueError: cannot view annotated file without creating it

    ::

        sage: collected_paths = []
        sage: cython('''
        ....: def f(int n):
        ....:     return n*n
        ....: ''', view_annotate=True, view_annotate_callback=collected_paths.append)
        sage: collected_paths
        ['...']
        sage: len(collected_paths)
        1
    """
    if view_annotate_callback is None:
        # needed because of https://github.com/sagemath/sage/pull/38946#issuecomment-2656329774
        view_annotate_callback = _webbrowser_open_file
    if not filename.endswith('pyx'):
        print("Warning: file (={}) should have extension .pyx".format(filename), file=sys.stderr)

    # base is the name of the .so module that we create. If we are
    # creating a local shared object file, we use a more natural
    # naming convention. If we are not creating a local shared object
    # file, the main constraint is that it is unique and determined by
    # the file that we're running Cython on, so that in some cases we
    # can cache the result (e.g., recompiling the same pyx file during
    # the same session).
    if create_local_so_file:
        base, ext = os.path.splitext(os.path.basename(filename))
    else:
        base = os.path.abspath(filename)
    base = sanitize(base)

    # This is the *temporary* directory where we store the pyx file.
    # spyx_tmp changes when we start Sage, so old (but not stale) pyx
    # files must be rebuilt at the moment.
    target_dir = os.path.join(spyx_tmp(), base)

    # Build directory for Cython/distutils
    build_dir = os.path.join(target_dir, "build")

    if os.path.exists(target_dir):
        # There is already a module here. Maybe we do not have to rebuild?
        # Find the name.
        if use_cache:
            from importlib.machinery import EXTENSION_SUFFIXES
            for f in os.listdir(target_dir):
                for suffix in EXTENSION_SUFFIXES:
                    if f.endswith(suffix):
                        # use the first matching extension
                        prev_file = os.path.join(target_dir, f)
                        prev_name = f[:-len(suffix)]
                        break
                else:
                    # no match, try next file
                    continue
                if os.path.getmtime(filename) <= os.path.getmtime(prev_file):
                    # We do not have to rebuild.
                    return prev_name, target_dir

        # Delete all ordinary files in target_dir
        for F in os.listdir(target_dir):
            G = os.path.join(target_dir, F)
            if os.path.isdir(G):
                continue
            try:
                os.unlink(G)
            except OSError:
                pass
    else:
        os.makedirs(target_dir, exist_ok=True)

    if create_local_so_file:
        name = base
    else:
        global sequence_number
        if base not in sequence_number:
            sequence_number[base] = 0
        name = '%s_%s' % (base, sequence_number[base])

        # increment the sequence number so will use a different one next time.
        sequence_number[base] += 1

    if compile_message:
        sys.stderr.write("Compiling {}...\n".format(filename))
        sys.stderr.flush()

    # Copy original file to the target directory.
    pyxfile = os.path.join(target_dir, name + ".pyx")
    shutil.copy(filename, pyxfile)

    # Add current working directory to includes. This is needed because
    # we cythonize from a different directory. See Issue #24764.
    standard_libs, standard_libdirs, standard_includes, aliases = _standard_libs_libdirs_incdirs_aliases()
    includes = [os.getcwd()] + standard_includes

    # Now do the actual build, directly calling Cython and distutils
    from Cython.Build import cythonize
    from Cython.Compiler.Errors import CompileError
    import Cython.Compiler.Options

    try:
        from setuptools.dist import Distribution
        from setuptools.extension import Extension
    except ImportError:
        # Fall back to distutils (stdlib); note that it is deprecated
        # in Python 3.10, 3.11; https://www.python.org/dev/peps/pep-0632/
        from distutils.dist import Distribution
        from distutils.core import Extension

    from distutils.log import set_verbosity
    set_verbosity(verbose)

    Cython.Compiler.Options.annotate = annotate
    Cython.Compiler.Options.embed_pos_in_docstring = True
    Cython.Compiler.Options.pre_import = "sage.all" if sage_namespace else None

    extra_compile_args = ['-w']  # no warnings
    extra_link_args = []

    ext = Extension(name,
                    sources=[pyxfile],
                    extra_compile_args=extra_compile_args,
                    extra_link_args=extra_link_args,
                    libraries=standard_libs,
                    library_dirs=standard_libdirs)

    directives = {'language_level': 3, 'cdivision': True}

    try:
        # Change directories to target_dir so that Cython produces the correct
        # relative path; https://github.com/sagemath/sage/issues/24097
        with restore_cwd(target_dir):
            try:
                from sage.misc.package_dir import cython_namespace_package_support
                with cython_namespace_package_support():
                    ext, = cythonize([ext],
                                     aliases=aliases,
                                     include_path=includes,
                                     compiler_directives=directives,
                                     quiet=(verbose <= 0),
                                     errors_to_stderr=False,
                                     use_listing_file=True)
            finally:
                # Read the "listing file" which is the file containing
                # warning and error messages generated by Cython.
                try:
                    with open(name + ".lis") as f:
                        cython_messages = f.read()
                except OSError:
                    cython_messages = "Error compiling Cython file"
    except CompileError:
        raise RuntimeError(cython_messages.strip())

    if verbose >= 0:
        # triggered by Cython 3 with unpatched cysignals 1.11.2
        cython_messages = re.sub(
            "^.*The keyword 'nogil' should appear at the end of the function signature line. "
            "Placing it before 'except' or 'noexcept' will be disallowed in a future version of Cython.\n",
            "", cython_messages, flags=re.MULTILINE)

        sys.stderr.write(cython_messages)
        sys.stderr.flush()

    if create_local_c_file:
        shutil.copy(os.path.join(target_dir, ext.sources[0]),
                    os.curdir)
        if annotate:
            shutil.copy(os.path.join(target_dir, name + ".html"),
                        os.curdir)

    if view_annotate:
        if not annotate:
            raise ValueError("cannot view annotated file without creating it")
        view_annotate_callback(os.path.join(target_dir, name + ".html"))

    # This emulates running "setup.py build" with the correct options
    #
    # setuptools plugins considered harmful:
    # If build isolation is not in use and setuptools_scm is installed,
    # then its file_finders entry point is invoked, which we don't need.
    # And with setuptools_scm 8, we get more trouble:
    # LookupError: pyproject.toml does not contain a tool.setuptools_scm section
    # LookupError: setuptools-scm was unable to detect version ...
    # We just remove all handling of "setuptools.finalize_distribution_options" entry points.
    class Distribution_no_finalize_distribution_options(Distribution):
        @staticmethod
        def _removed(ep):
            return True

    dist = Distribution_no_finalize_distribution_options()
    dist.ext_modules = [ext]
    dist.include_dirs = includes
    buildcmd = dist.get_command_obj("build")
    buildcmd.build_base = build_dir
    buildcmd.build_lib = target_dir

    try:
        # Capture errors from distutils and its child processes
        with open(os.path.join(target_dir, name + ".err"), 'w+') as errfile:
            try:
                # Redirect stderr to errfile.  We use the file descriptor
                # number "2" instead of "sys.stderr" because we really
                # want to redirect the messages from GCC. These are sent
                # to the actual stderr, regardless of what sys.stderr is.
                sys.stderr.flush()
                with redirection(2, errfile, close=False):
                    dist.run_command("build")
            finally:
                errfile.seek(0)
                distutils_messages = errfile.read()
    except Exception as msg:
        msg = str(msg) + "\n" + distutils_messages
        raise RuntimeError(msg.strip())

    if verbose >= 0:
        sys.stderr.write(distutils_messages)
        sys.stderr.flush()

    if create_local_so_file:
        # Copy module to current directory
        from importlib.machinery import EXTENSION_SUFFIXES
        for ext in EXTENSION_SUFFIXES:
            path = os.path.join(target_dir, name + ext)
            if os.path.exists(path):
                shutil.copy(path, os.curdir)

    return name, target_dir


################################################################
# COMPILE
################################################################
def cython_lambda(vars, expr, verbose=0, **kwds):
    """
    Create a compiled function which evaluates ``expr`` assuming machine values
    for ``vars``.

    INPUT:

    - ``vars`` -- list of pairs (variable name, c-data type), where the variable
      names and data types are strings, OR a string such as ``'double x, int y,
      int z'``

    - ``expr`` -- an expression involving the vars and constants; you can access
      objects defined in the current module scope ``globals()`` using
      ``sage.object_name``.

    .. warning::

        Accessing ``globals()`` doesn't actually work, see :issue:`12446`.

    EXAMPLES:

    We create a Lambda function in pure Python (using the r to make sure the 3.2
    is viewed as a Python float)::

        sage: f = lambda x,y: x*x + y*y + x + y + 17r*x + 3.2r

    We make the same Lambda function, but in a compiled form. ::

        sage: g = cython_lambda('double x, double y', 'x*x + y*y + x + y + 17*x + 3.2')
        sage: g(2,3)
        55.2
        sage: g(0,0)
        3.2

    In order to access Sage globals, prefix them with ``sage.``::

        sage: f = cython_lambda('double x', 'sage.sin(x) + sage.a')
        sage: f(0)
        Traceback (most recent call last):
        ...
        NameError: global 'a' is not defined
        sage: a = 25
        sage: f(10)
        24.45597888911063
        sage: a = 50
        sage: f(10)
        49.45597888911063
    """
    if isinstance(vars, str):
        v = vars
    else:
        v = ', '.join('%s %s' % (typ, var) for typ, var in vars)

    s = """
cdef class _s:
    cdef globals

    def __init__(self):
        from sage.repl.user_globals import get_globals
        self.globals = get_globals()

    def __getattr__(self, name):
        try:
            return self.globals[name]
        except KeyError:
            raise NameError("global {!r} is not defined".format(name))

sage = _s()

def f(%s):
    return %s
    """ % (v, expr)
    if verbose > 0:
        print(s)
    tmpfile = tmp_filename(ext='.pyx')
    with open(tmpfile, 'w') as f:
        f.write(s)

    d = {}
    cython_import_all(tmpfile, d, verbose=verbose, **kwds)
    return d['f']


################################################################
# IMPORT
################################################################
def cython_import(filename, **kwds):
    """
    Compile a file containing Cython code, then import and return the
    module.  Raises an :exc:`ImportError` if anything goes wrong.

    INPUT:

    - ``filename`` -- string; name of a file that contains Cython
      code

    See the function :func:`sage.misc.cython.cython` for documentation
    for the other inputs.

    OUTPUT: the module that contains the compiled Cython code
    """
    name, build_dir = cython(filename, **kwds)

    oldpath = sys.path
    try:
        sys.path.append(build_dir)
        return builtins.__import__(name)
    except ModuleNotFoundError:
        import importlib
        importlib.invalidate_caches()
        return builtins.__import__(name)
    finally:
        sys.path = oldpath


def cython_import_all(filename, globals, **kwds):
    """
    Imports all non-private (i.e., not beginning with an underscore)
    attributes of the specified Cython module into the given context.
    This is similar to::

        from module import *

    Raises an :exc:`ImportError` exception if anything goes wrong.

    INPUT:

    - ``filename`` -- string; name of a file that contains Cython
      code

    See the function :func:`sage.misc.cython.cython` for documentation
    for the other inputs.
    """
    m = cython_import(filename, **kwds)
    for k, x in m.__dict__.items():
        if k[0] != '_':
            globals[k] = x


def sanitize(f):
    """
    Given a filename ``f``, replace it by a filename that is a valid Python
    module name.

    This means that the characters are all alphanumeric or ``_``'s and doesn't
    begin with a numeral.

    EXAMPLES::

        sage: from sage.misc.cython import sanitize
        sage: sanitize('abc')
        'abc'
        sage: sanitize('abc/def')
        'abc_def'
        sage: sanitize('123/def-hij/file.py')
        '_123_def_hij_file_py'
    """
    s = ''
    if f[0].isdigit():
        s += '_'
    for a in f:
        if a.isalnum():
            s += a
        else:
            s += '_'
    return s


def compile_and_load(code, **kwds):
    r"""
    INPUT:

    - ``code`` -- string containing code that could be in a .pyx file
      that is attached or put in a %cython block in the notebook

    See the function :func:`sage.misc.cython.cython` for documentation
    for the other inputs.

    OUTPUT: a module, which results from compiling the given code and
    importing it

    EXAMPLES::

        sage: from sage.misc.cython import compile_and_load
        sage: module = compile_and_load("def f(int n):\n    return n*n")
        sage: module.f(10)
        100

    TESTS::

        sage: code = '''
        ....: from sage.rings.rational cimport Rational
        ....: from sage.rings.polynomial.polynomial_rational_flint cimport Polynomial_rational_flint
        ....: from sage.libs.flint.fmpq_poly cimport fmpq_poly_length
        ....: from sage.libs.flint.fmpq_poly_sage cimport fmpq_poly_get_coeff_mpq, fmpq_poly_set_coeff_mpq
        ....:
        ....: def evaluate_at_power_of_gen(Polynomial_rational_flint f, unsigned long n):
        ....:     assert n >= 1
        ....:     cdef Polynomial_rational_flint res = f._new()
        ....:     cdef unsigned long k
        ....:     cdef Rational z = Rational(0)
        ....:     for k in range(fmpq_poly_length(f._poly)):
        ....:         fmpq_poly_get_coeff_mpq(z.value, f._poly, k)
        ....:         fmpq_poly_set_coeff_mpq(res._poly, n*k, z.value)
        ....:     return res
        ....: '''
        sage: module = compile_and_load(code)  # long time
        sage: R.<x> = QQ[]
        sage: module.evaluate_at_power_of_gen(x^3 + x - 7, 5)  # long time
        x^15 + x^5 - 7
    """
    tmpfile = tmp_filename(ext='.pyx')
    with open(tmpfile, 'w') as f:
        f.write(code)
    return cython_import(tmpfile, **kwds)


def cython_compile(code, **kwds):
    """
    Given a block of Cython code (as a text string), this function
    compiles it using a C compiler, and includes it into the global
    namespace.

    AUTHOR: William Stein, 2006-10-31

    .. WARNING::

        Only use this from Python code, not from extension code, since
        from extension code you would change the global scope (i.e.,
        of the Sage interpreter). And it would be stupid, since you're
        already writing Cython!

        Also, never use this in the standard Sage library.  Any code
        that uses this can only run on a system that has a C compiler
        installed, and we want to avoid making that assumption for
        casual Sage usage.  Also, any code that uses this in the
        library would greatly slow down startup time, since currently
        there is no caching.

    .. TODO::

        Need to create a clever caching system so code only gets
        compiled once.
    """
    tmpfile = tmp_filename(ext='.pyx')
    with open(tmpfile, 'w') as f:
        f.write(code)
    return cython_import_all(tmpfile, get_globals(), **kwds)



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.graphs sage.numerical.mip sage.symbolic
"""
This file (./lp_doctest.sage) was *autogenerated* from ./lp.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./lp_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./lp.tex, line 14::

  sage: set_random_seed(158888)
  sage: sage.numerical.backends.generic_backend.default_solver = "Glpk"

Sage example in ./lp.tex, line 124::

  sage: p = MixedIntegerLinearProgram()

Sage example in ./lp.tex, line 137::

  sage: x, y, z = p['x'], p['y'], p['z']
  sage: p.set_min(x, 0)
  sage: p.set_min(y, 0)
  sage: p.set_min(z, 0)

Sage example in ./lp.tex, line 154::

  sage: p.set_objective( x + y + 3*z )

Sage example in ./lp.tex, line 166::

  sage: p.add_constraint( x + 2*y <= 4 )
  sage: p.add_constraint( 5*z - y  <= 8 )

Sage example in ./lp.tex, line 180::

  sage: p.solve() # abs tol 1e-10
  8.8

Sage example in ./lp.tex, line 194::

  sage: p.get_values(x), p.get_values(y), p.get_values(z) # abs tol 1e-10
  (4.0, 0.0, 1.6)

Sage example in ./lp.tex, line 214::

  sage: x = p.new_variable()

Sage example in ./lp.tex, line 224::

  sage: p.add_constraint( x[1] + x[12] - x[14] >= 8 )

Sage example in ./lp.tex, line 236::

  sage: p.add_constraint( x["i_am_a_valid_index"] + x["a",pi] <= 3 )

Sage example in ./lp.tex, line 248::

  sage: p.add_constraint( p.sum(
  ....:       x[i,j] for i in range(4) for j in range(4)) <= 1 )

Sage example in ./lp.tex, line 287::

  sage: p = MixedIntegerLinearProgram()
  sage: p.set_objective( p[3] + p[2] )
  sage: p.add_constraint( p[3] <= 5 )

Sage example in ./lp.tex, line 293::

  sage: p.solve()
  Traceback (most recent call last):
    ...
  MIPSolverException: GLPK: The LP (relaxation) problem has no dual feasible solution

Sage example in ./lp.tex, line 301::

  sage: p.add_constraint( p[2] <= 8 )
  sage: p.solve() # abs tol 1e-10
  13.0

Sage example in ./lp.tex, line 306::

  sage: p.add_constraint( p[3] >= 6 ); p.solve()
  Traceback (most recent call last):
    ...
  MIPSolverException: GLPK: Problem has no feasible solution

Sage example in ./lp.tex, line 319::

  sage: p = MixedIntegerLinearProgram()
  sage: p.set_objective( p[3] )
  sage: p.add_constraint( p[3] <= 4.75 ); p.add_constraint( p[3] >= 4.25 )
  sage: p.solve() # abs tol 1e-10
  4.75
  sage: p.set_integer(p[3]); p.solve()
  Traceback (most recent call last):
    ...
  MIPSolverException: GLPK: Problem has no feasible solution

Sage example in ./lp.tex, line 360::

  sage: try:
  ....:    p.solve()
  ....:    print("The problem has a solution!")
  ....: except RuntimeError:
  ....:    print("The problem is infeasible!")
  The problem is infeasible!

Sage example in ./lp.tex, line 429::

  sage: C = 1
  sage: L = ["Pan", "Book", "Knife", "Flask", "Flashlight"]
  sage: w = [0.57,0.35,0.98,0.39,0.08]; u = [0.57,0.26,0.29,0.85,0.23]
  sage: weight = {}; utility = {}
  sage: for o in L:
  ....:    weight[o] = w[L.index(o)]; utility[o] = u[L.index(o)]

Sage example in ./lp.tex, line 451::

  sage: p = MixedIntegerLinearProgram()
  sage: taken = p.new_variable( binary = True )
  sage: p.add_constraint(
  ....:   p.sum( weight[o] * taken[o] for o in L ) <= C )
  sage: p.set_objective(
  ....:   p.sum( utility[o] * taken[o] for o in L ) )
  sage: p.solve() # abs tol 1e-10
  1.4199999999999999
  sage: taken = p.get_values(taken)

Sage example in ./lp.tex, line 470::

  sage: sum( weight[o] * taken[o] for o in L ) # abs tol 1e-10
  0.960000000000000

Sage example in ./lp.tex, line 482::

  sage: taken["Flask"]
  1.0

Sage example in ./lp.tex, line 551::

  sage: g = graphs.PetersenGraph()
  sage: p = MixedIntegerLinearProgram()
  sage: matching = p.new_variable(binary = True)

Sage example in ./lp.tex, line 561::

  sage: p.set_objective(p.sum(matching[e]
  ....:                 for e in g.edges(sort=True, labels = False)))

Sage example in ./lp.tex, line 573::

  sage: for v in g:
  ....:    p.add_constraint(p.sum(matching[e]
  ....:          for e in g.edges_incident(v, labels = False)) <= 1)
  sage: p.solve()
  5.0

Sage example in ./lp.tex, line 586::

  sage: matching = p.get_values(matching, convert=ZZ, tolerance=1e-3)
  sage: sorted(e for e, b in matching.items() if b == 1)
  [(0, 1), (2, 3), (4, 9), (5, 7), (6, 8)]

Sage example in ./lp.tex, line 665::

  sage: g = graphs.ChvatalGraph()
  sage: g = g.minimum_outdegree_orientation()

Sage example in ./lp.tex, line 669::

  sage: p = MixedIntegerLinearProgram()
  sage: f = p.new_variable()
  sage: s, t = 0, 2

Sage example in ./lp.tex, line 674::

  sage: for v in g:
  ....:    if v == s or v == t: continue
  ....:    p.add_constraint(
  ....:          p.sum(f[v,u] for u in g.neighbors_out(v)) ==
  ....:          p.sum(f[u,v] for u in g.neighbors_in(v)))

Sage example in ./lp.tex, line 681::

  sage: for e in g.edges(sort=True, labels = False): p.add_constraint( f[e] <= 1 )

Sage example in ./lp.tex, line 684::

  sage: p.set_objective(p.sum( f[s,u] for u in g.neighbors_out(s)))

Sage example in ./lp.tex, line 687::

  sage: p.solve()
  2.0

Sage example in ./lp.tex, line 847::

  sage: g = graphs.Grid2dGraph(4, 6)
  sage: p = MixedIntegerLinearProgram()
  sage: b = p.new_variable(binary = True)

Sage example in ./lp.tex, line 861::

  sage: B = lambda x,y : b[frozenset([x,y])]

Sage example in ./lp.tex, line 874::

  sage: for u in g:
  ....:   p.add_constraint( p.sum( B(u,v) for v in g.neighbors(u) ) == 2 )

Sage example in ./lp.tex, line 892::

  sage: p.solve()
  0.0
  sage: h = Graph()
  sage: h.add_edges( [(u,v) for u, v in g.edges(sort=True, labels = False)
  ....:                if p.get_values(B(u,v), convert=ZZ, tolerance=1e-3) == 1] )

Sage example in ./lp.tex, line 906::

  sage: while not h.is_connected():
  ....:     S = h.connected_components(sort=False)[0]
  ....:     p.add_constraint(
  ....:            p.sum( B(u,v) for u,v
  ....:                 in g.edge_boundary(S, labels = False))
  ....:                >= 2)
  ....:     zero = p.solve()
  ....:     h = Graph()
  ....:     h.add_edges( [(u,v)
  ....:                   for u,v in g.edges(sort=True, labels = False)
  ....:                   if p.get_values(B(u,v), convert=ZZ, tolerance=1e-3) == 1] )
"""



# sage_setup: distribution = sagemath-repl
"""
Helper for attaching tolerance information to strings
"""

# ****************************************************************************
#       Copyright (C) 2012-2018 David Roe <roed.math@gmail.com>
#                     2012      Robert Bradshaw <robertwb@gmail.com>
#                     2012      William Stein <wstein@gmail.com>
#                     2013      R. Andrew Ohana
#                     2013      Volker Braun
#                     2013-2018 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2016-2021 Frédéric Chapoton
#                     2017-2018 Erik M. Bray
#                     2020      Marc Mezzarobba
#                     2020-2023 Matthias Koeppe
#                     2022      John H. Palmieri
#                     2022      Sébastien Labbé
#                     2023      Kwankyu Lee
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************


class MarkedOutput(str):
    """
    A subclass of string with context for whether another string
    matches it.

    EXAMPLES::

        sage: from sage.doctest.marked_output import MarkedOutput
        sage: s = MarkedOutput("abc")
        sage: s.rel_tol
        0
        sage: s.update(rel_tol = .05)
        'abc'
        sage: s.rel_tol
        0.0500000000000000

        sage: MarkedOutput("56 µs")
        '56 \xb5s'
    """
    random = False
    rel_tol = 0
    abs_tol = 0
    tol = 0

    def update(self, **kwds):
        """
        EXAMPLES::

            sage: from sage.doctest.marked_output import MarkedOutput
            sage: s = MarkedOutput("0.0007401")
            sage: s.update(abs_tol = .0000001)
            '0.0007401'
            sage: s.rel_tol
            0
            sage: s.abs_tol
            1.00000000000000e-7
        """
        self.__dict__.update(kwds)
        return self

    def __reduce__(self):
        """
        Pickling.

        EXAMPLES::

            sage: from sage.doctest.marked_output import MarkedOutput
            sage: s = MarkedOutput("0.0007401")
            sage: s.update(abs_tol = .0000001)
            '0.0007401'
            sage: t = loads(dumps(s)) # indirect doctest
            sage: t == s
            True
            sage: t.abs_tol
            1.00000000000000e-7
        """
        return make_marked_output, (str(self), self.__dict__)


def make_marked_output(s, D):
    """
    Auxiliary function for pickling.

    EXAMPLES::

        sage: from sage.doctest.marked_output import make_marked_output
        sage: s = make_marked_output("0.0007401", {'abs_tol':.0000001})
        sage: s
        '0.0007401'
        sage: s.abs_tol
        1.00000000000000e-7
    """
    ans = MarkedOutput(s)
    ans.__dict__.update(D)
    return ans
